# Copyright (c) 2021 J. Lewis Muir
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#
# An abstract class for an application.
#
# The methods that must be overridden are run and usage_message.
#
class Application
  # Escapes characters for safely displaying to a terminal.
  def self.terminal_escape(obj)
    obj.to_s.gsub(/\p{Cc}|\\/) do |m|
      m == '\\' ? '\\\\' : m.bytes.map { |b| sprintf('\x%02x', b) }.join
    end
  end

  def initialize(
    prog_name: File.basename($0),
    argv: ARGV,
    stdin: $stdin,
    stdout: $stdout,
    stderr: $stderr,
    env: ENV
  )
    @prog_name = prog_name
    @argv = argv
    @stdin = stdin
    @stdout = stdout
    @stderr = stderr
    @env = env
  end

  def run
    raise(NotImplementedError, 'Need to define "run" method')
  end

  protected

  def errx(status, msg)
    @stderr.puts("#{@prog_name}: #{msg}")
    exit(status)
  end

  def warnx(msg)
    @stderr.puts("#{@prog_name}: #{msg}")
  end

  def usage
    @stderr.puts(usage_message)
    exit(1)
  end

  def usage_message
    raise(NotImplementedError, 'Need to define "usage_message" method')
  end
end
